@@ -346,7 +346,7 @@ tracker_data_update_get_new_service_id (void)
 	g_object_unref (stmt);
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor);
+		tracker_db_cursor_iter_next (cursor, NULL);
 		max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
 		g_object_unref (cursor);
 	}
@@ -380,7 +380,7 @@ tracker_data_update_get_next_modseq (void)
 	g_object_unref (stmt);
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor);
+		tracker_db_cursor_iter_next (cursor, NULL);
 		max_modseq = MAX (tracker_db_cursor_get_int (cursor, 0), max_modseq);
 		g_object_unref (cursor);
 	}
@@ -1325,35 +1325,13 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		g_value_unset (&gvalue);
 	} else if (!multiple_values && old_values->n_values > 1) {
 		/* trying to add second value to single valued property */
-		GValue old_value = { 0 };
-		GValue new_value = { 0 };
-		const gchar *old_value_str = NULL;
-		const gchar *new_value_str = NULL;
 
-		g_value_init (&old_value, G_TYPE_STRING);
-		g_value_init (&new_value, G_TYPE_STRING);
-
-		/* Get both old and new values as strings letting glib do
-		 * whatever transformation needed */
-		if (g_value_transform (g_value_array_get_nth (old_values, 0), &old_value)) {
-			old_value_str = g_value_get_string (&old_value);
-		}
-		if (g_value_transform (g_value_array_get_nth (old_values, 1), &new_value)) {
-			new_value_str = g_value_get_string (&new_value);
-		}
-
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
-		             "Unable to insert multiple values for subject `%s' and single valued property `%s' "
-		             "(old_value: '%s', new value: '%s')",
-		             resource_buffer->subject,
-		             field_name,
-		             old_value_str ? old_value_str : "<untransformable>",
-		             new_value_str ? new_value_str : "<untransformable>");
-
-		g_value_unset (&old_value);
-		g_value_unset (&new_value);
 		g_value_unset (&gvalue);
 
+		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
+		             "Unable to insert multiple values for subject `%s' and single valued property `%s'",
+		             resource_buffer->subject,
+		             field_name);
 	} else {
 		cache_insert_value (table_name, field_name, &gvalue,
 		                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
@@ -2013,6 +1991,12 @@ tracker_data_commit_db_transaction (void)
 	g_hash_table_remove_all (update_buffer.resources_by_id);
 	g_hash_table_remove_all (update_buffer.resource_cache);
 
+	in_journal_replay = FALSE;
+}
+
+void
+tracker_data_notify_db_transaction (void)
+{
 	if (commit_callbacks) {
 		guint n;
 		for (n = 0; n < commit_callbacks->len; n++) {
@@ -2021,10 +2005,9 @@ tracker_data_commit_db_transaction (void)
 			delegate->callback (delegate->user_data);
 		}
 	}
-
-	in_journal_replay = FALSE;
 }
 
+
 static void
 format_sql_value_as_string (GString         *sql,
                             TrackerProperty *property)
@@ -2377,7 +2360,7 @@ free_queued_statement (QueuedStatement *queued)
 }
 
 static GList*
-queue_statement (GList *queue,
+queue_statement (GList *queue, 
                  const gchar *graph,
                  const gchar *subject,
                  const gchar *predicate,
@@ -2421,10 +2404,10 @@ ontology_transaction_end (GList *ontology_queue,
 
 		/* store ontology in database */
 		tracker_data_ontology_process_statement (queued->graph,
-		                                         queued->subject,
-		                                         queued->predicate,
-		                                         queued->object,
-		                                         queued->is_uri,
+		                                         queued->subject, 
+		                                         queued->predicate, 
+		                                         queued->object, 
+		                                         queued->is_uri, 
 		                                         TRUE, TRUE);
 
 	}
