@@ -58,6 +58,8 @@ struct _TrackerPropertyPrivate {
 	gboolean       transient;
 	gboolean       is_inverse_functional_property;
 	gboolean       is_new;
+	gboolean       db_schema_changed;
+	gboolean       writeback;
 
 	GArray        *super_properties;
 };
@@ -348,6 +350,30 @@ tracker_property_get_is_new (TrackerProperty *property)
 	return priv->is_new;
 }
 
+gboolean
+tracker_property_get_writeback (TrackerProperty *property)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_PROPERTY (property), FALSE);
+
+	priv = GET_PRIV (property);
+
+	return priv->writeback;
+}
+
+gboolean
+tracker_property_get_db_schema_changed (TrackerProperty *property)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_PROPERTY (property), FALSE);
+
+	priv = GET_PRIV (property);
+
+	return priv->db_schema_changed;
+}
+
 gboolean
 tracker_property_get_embedded (TrackerProperty *property)
 {
@@ -558,6 +584,32 @@ tracker_property_set_is_new (TrackerProperty *property,
 	priv->is_new = value;
 }
 
+void
+tracker_property_set_writeback (TrackerProperty *property,
+                                gboolean         value)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+
+	priv = GET_PRIV (property);
+
+	priv->writeback = value;
+}
+
+void
+tracker_property_set_db_schema_changed (TrackerProperty *property,
+                                        gboolean         value)
+{
+	TrackerPropertyPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+
+	priv = GET_PRIV (property);
+
+	priv->db_schema_changed = value;
+}
+
 void
 tracker_property_set_fulltext_indexed (TrackerProperty *property,
                                        gboolean         value)
