@@ -46,6 +46,30 @@
 #define RDF_PREFIX TRACKER_RDF_PREFIX
 #define RDF_TYPE RDF_PREFIX "type"
 
+/* I *know* that this is some arbitrary number that doesn't seem to
+ * resemble anything. In fact it's what I experimentally measured to
+ * be a good value on a default Debian testing which has
+ * max_message_size set to 1 000 000 000 in session.conf. I didn't have
+ * the feeling that this value was very much respected, as the size
+ * of the DBusMessage when libdbus decided to exit() the process was
+ * around 160 MB, and not ~ 1000 MB. So if you take 160 MB and you
+ * devide it by 1000000 you have an average string size of ~ 160
+ * bytes plus DBusMessage's overhead. If that makes this number less
+ * arbitrary for you, then fine.
+ *
+ * I really hope that the libdbus people get to their senses and 
+ * either stop doing their exit() nonsense in a library, and instead
+ * return a clean DBusError or something, or create crystal clear
+ * clarity about the maximum size of a message. And make it both so
+ * that I can get this length at runtime (without having to parse
+ * libdbus's own configuration files) and my DBusMessage's current
+ * total length. As far as I know are both not possible. So that for
+ * me means that libdbus's exit() is unacceptable.
+ *
+ * Note for the debugger of the future, the "Disconnected" signal gets
+ * sent to us by the bus, which in turn makes libdbus-glib perform exit(). */
+
+#define DBUS_ARBITRARY_MAX_MSG_SIZE 1000000
 
 G_DEFINE_TYPE(TrackerResources, tracker_resources, G_TYPE_OBJECT)
 
@@ -63,8 +87,14 @@ typedef struct {
 typedef struct {
 	DBusGMethodInvocation *context;
 	guint request_id;
+	DBusMessage *reply;
 } TrackerDBusMethodInfo;
 
+typedef struct {
+	DBusMessage *reply;
+	GError *error;
+	gpointer user_data;
+} InThreadPtr;
 
 static void tracker_resources_finalize (GObject *object);
 
@@ -152,7 +182,7 @@ tracker_resources_load (TrackerResources         *object,
                         GError                  **error)
 {
 	TrackerDBusMethodInfo   *info;
-	guint               request_id;
+	guint                    request_id;
 	GFile  *file;
 
 	request_id = tracker_dbus_get_next_request_id ();
@@ -178,16 +208,120 @@ tracker_resources_load (TrackerResources         *object,
 	g_object_unref (file);
 }
 
+static void
+query_callback (gpointer inthread_data, GError *error, gpointer user_data)
+{
+	InThreadPtr *ptr = inthread_data;
+	TrackerDBusMethodInfo *info = user_data;
+
+	if (ptr->error) {
+		tracker_dbus_request_failed (info->request_id,
+		                             info->context,
+		                             &ptr->error,
+		                             NULL);
+		dbus_g_method_return_error (info->context, ptr->error);
+		g_error_free (ptr->error);
+	} else {
+
+		tracker_dbus_request_success (info->request_id,
+		                              info->context);
+
+		dbus_g_method_send_reply (info->context, ptr->reply);
+	}
+
+	g_slice_free (InThreadPtr, ptr);
+}
+
+static gpointer
+query_inthread (TrackerDBCursor *cursor, GError *error, gpointer user_data)
+{
+	InThreadPtr *ptr = g_slice_new0 (InThreadPtr);
+	TrackerDBusMethodInfo *info = user_data;
+	DBusMessage *reply;
+	DBusMessageIter iter, rows_iter;
+	guint cols;
+	GError *loop_error = NULL;
+	guint length = 0;
+	gboolean cont;
+
+	if (error) {
+		ptr->error = g_error_copy (error);
+		return ptr;
+	}
+
+	reply = info->reply;
+
+	dbus_message_iter_init_append (reply, &iter);
+
+	cols = tracker_db_cursor_get_n_columns (cursor);
+
+	dbus_message_iter_open_container (&iter,
+	                                  DBUS_TYPE_ARRAY,
+	                                  "as",
+	                                  &rows_iter);
+
+	cont = TRUE;
+
+	while (tracker_db_cursor_iter_next (cursor, &loop_error) && cont) {
+		DBusMessageIter cols_iter;
+		guint i;
+
+		if (loop_error != NULL) {
+			break;
+		}
+
+		dbus_message_iter_open_container (&rows_iter,
+		                                  DBUS_TYPE_ARRAY,
+		                                  "s",
+		                                  &cols_iter);
+
+		for (i = 0; i < cols && cont; i++, length++) {
+			const gchar *result_str;
+			result_str = tracker_db_cursor_get_string (cursor, i);
+
+			if (result_str == NULL)
+				result_str = "";
+
+			dbus_message_iter_append_basic (&cols_iter, DBUS_TYPE_STRING, &result_str);
+
+			if (length > DBUS_ARBITRARY_MAX_MSG_SIZE) {
+				g_set_error (&loop_error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_INTERRUPTED,
+				             "result set of the query is too large");
+
+				cont = FALSE;
+			}
+
+		}
+
+		dbus_message_iter_close_container (&rows_iter, &cols_iter);
+	}
+
+	dbus_message_iter_close_container (&iter, &rows_iter);
+
+	if (loop_error) {
+		ptr->error = loop_error;
+		ptr->reply = NULL;
+	} else {
+		ptr->reply = reply;
+	}
+
+	return ptr;
+}
+
 void
 tracker_resources_sparql_query (TrackerResources         *self,
                                 const gchar              *query,
                                 DBusGMethodInvocation    *context,
                                 GError                  **error)
 {
-	TrackerDBResultSet   *result_set;
-	GError               *actual_error = NULL;
+	TrackerDBusMethodInfo   *info;
+	TrackerResourcesPrivate *priv;
 	guint                 request_id;
-	GPtrArray            *values;
+	gchar                 *sender;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
 
 	request_id = tracker_dbus_get_next_request_id ();
 
@@ -199,28 +333,38 @@ tracker_resources_sparql_query (TrackerResources         *self,
 	                          __FUNCTION__,
 	                          query);
 
-	result_set = tracker_store_sparql_query (query, &actual_error);
+	info = g_slice_new (TrackerDBusMethodInfo);
 
-	if (actual_error) {
-		tracker_dbus_request_failed (request_id,
-		                             context,
-		                             &actual_error,
+	info->request_id = request_id;
+	info->context = context;
+	info->reply = dbus_g_method_get_reply (context);
+
+	sender = dbus_g_method_get_sender (context);
+
+	tracker_store_sparql_query (query, TRACKER_STORE_PRIORITY_HIGH,
+	                            query_inthread, query_callback, sender,
+	                            info, destroy_method_info);
+
+	g_free (sender);
+}
+
+static void
+update_callback (GError *error, gpointer user_data)
+{
+	TrackerDBusMethodInfo *info = user_data;
+
+	if (error) {
+		tracker_dbus_request_failed (info->request_id,
+		                             info->context,
+		                             &error,
 		                             NULL);
-		dbus_g_method_return_error (context, actual_error);
-		g_error_free (actual_error);
+		dbus_g_method_return_error (info->context, error);
 		return;
 	}
 
-	values = tracker_dbus_query_result_to_ptr_array (result_set);
-
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, values);
-
-	tracker_dbus_results_ptr_array_free (&values);
-
-	if (result_set) {
-		g_object_unref (result_set);
-	}
+	tracker_dbus_request_success (info->request_id,
+	                              info->context);
+	dbus_g_method_return (info->context);
 }
 
 void
@@ -229,9 +373,10 @@ tracker_resources_sparql_update (TrackerResources        *self,
                                  DBusGMethodInvocation   *context,
                                  GError                 **error)
 {
+	TrackerDBusMethodInfo   *info;
 	TrackerResourcesPrivate *priv;
-	GError                       *actual_error = NULL;
 	guint                 request_id;
+	gchar                 *sender;
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
 
@@ -245,20 +390,37 @@ tracker_resources_sparql_update (TrackerResources        *self,
 	                          __FUNCTION__,
 	                          update);
 
-	tracker_store_sparql_update (update, &actual_error);
+	info = g_slice_new (TrackerDBusMethodInfo);
 
-	if (actual_error) {
-		tracker_dbus_request_failed (request_id,
-		                             context,
-		                             &actual_error,
+	info->request_id = request_id;
+	info->context = context;
+
+	sender = dbus_g_method_get_sender (context);
+
+	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_HIGH, FALSE,
+	                             update_callback, sender,
+	                             info, destroy_method_info);
+
+	g_free (sender);
+}
+
+static void
+update_blank_callback (GPtrArray *blank_nodes, GError *error, gpointer user_data)
+{
+	TrackerDBusMethodInfo *info = user_data;
+
+	if (error) {
+		tracker_dbus_request_failed (info->request_id,
+		                             info->context,
+		                             &error,
 		                             NULL);
-		dbus_g_method_return_error (context, actual_error);
-		g_error_free (actual_error);
+		dbus_g_method_return_error (info->context, error);
 		return;
 	}
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context);
+	tracker_dbus_request_success (info->request_id,
+	                              info->context);
+	dbus_g_method_return (info->context, blank_nodes);
 }
 
 void
@@ -267,10 +429,10 @@ tracker_resources_sparql_update_blank (TrackerResources       *self,
                                        DBusGMethodInvocation  *context,
                                        GError                **error)
 {
+	TrackerDBusMethodInfo   *info;
 	TrackerResourcesPrivate *priv;
-	GError                       *actual_error = NULL;
 	guint                 request_id;
-	GPtrArray            *blank_nodes;
+	gchar                 *sender;
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
 
@@ -284,35 +446,18 @@ tracker_resources_sparql_update_blank (TrackerResources       *self,
 	                          __FUNCTION__,
 	                          update);
 
-	blank_nodes = tracker_store_sparql_update_blank (update, &actual_error);
+	info = g_slice_new (TrackerDBusMethodInfo);
 
-	if (actual_error) {
-		tracker_dbus_request_failed (request_id,
-		                             context,
-		                             &actual_error,
-		                             NULL);
-		dbus_g_method_return_error (context, actual_error);
-		g_error_free (actual_error);
-		return;
-	}
+	info->request_id = request_id;
+	info->context = context;
 
-	if (!blank_nodes) {
-		/* Create empty GPtrArray for dbus-glib to be happy */
-		blank_nodes = g_ptr_array_new ();
-	}
+	sender = dbus_g_method_get_sender (context);
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, blank_nodes);
+	tracker_store_sparql_update_blank (update, TRACKER_STORE_PRIORITY_HIGH,
+	                                   update_blank_callback, sender,
+	                                   info, destroy_method_info);
 
-	if (blank_nodes) {
-		gint i;
-
-		for (i = 0; i < blank_nodes->len; i++) {
-			g_ptr_array_foreach (blank_nodes->pdata[i], (GFunc) g_hash_table_unref, NULL);
-			g_ptr_array_free (blank_nodes->pdata[i], TRUE);
-		}
-		g_ptr_array_free (blank_nodes, TRUE);
-	}
+	g_free (sender);
 }
 
 void
@@ -335,25 +480,6 @@ tracker_resources_sync (TrackerResources        *self,
 	dbus_g_method_return (context);
 }
 
-static void
-batch_update_callback (GError *error, gpointer user_data)
-{
-	TrackerDBusMethodInfo *info = user_data;
-
-	if (error) {
-		tracker_dbus_request_failed (info->request_id,
-		                             info->context,
-		                             &error,
-		                             NULL);
-		dbus_g_method_return_error (info->context, error);
-		return;
-	}
-
-	tracker_dbus_request_success (info->request_id,
-	                              info->context);
-	dbus_g_method_return (info->context);
-}
-
 void
 tracker_resources_batch_sparql_update (TrackerResources          *self,
                                        const gchar               *update,
@@ -384,8 +510,9 @@ tracker_resources_batch_sparql_update (TrackerResources          *self,
 
 	sender = dbus_g_method_get_sender (context);
 
-	tracker_store_queue_sparql_update (update, batch_update_callback,
-	                                   sender, info, destroy_method_info);
+	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_LOW, TRUE,
+	                             update_callback, sender,
+	                             info, destroy_method_info);
 
 	g_free (sender);
 }
